"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerFSEventHandlers = void 0;
const core_1 = require("@github-vsc-runner/core");
const fs_1 = require("fs");
const dayjs_1 = __importDefault(require("dayjs"));
const path_1 = __importDefault(require("path"));
const logger_1 = __importDefault(require("../logger"));
const foundation_1 = require("./foundation");
const search_1 = require("./search");
const stat = (path, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const filePath = foundation_1.resolveUri(path, cwd);
    const stats = yield fs_1.promises.stat(filePath);
    const { ctime, mtime, size } = stats;
    const data = {
        type: foundation_1.getFileType(stats),
        ctime: dayjs_1.default(ctime).unix(),
        mtime: dayjs_1.default(mtime).unix(),
        size,
    };
    return data;
});
const readDirectory = (dir, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const dirPath = foundation_1.resolveUri(dir, cwd);
    const files = yield fs_1.promises.readdir(dirPath);
    return yield Promise.all(files.map((file) => __awaiter(void 0, void 0, void 0, function* () {
        const stats = yield fs_1.promises.stat(path_1.default.join(dirPath, file));
        return [file, foundation_1.getFileType(stats)];
    })));
});
const readFile = (path, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const filePath = foundation_1.resolveUri(path, cwd);
    return fs_1.promises.readFile(filePath, 'utf-8');
});
const createDirectory = (path, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const filePath = foundation_1.resolveUri(path, cwd);
    return fs_1.promises.mkdir(filePath);
});
const writeFile = ({ uri, base64Content, options: { create, overwrite } }, cwd) => {
    const filePath = foundation_1.resolveUri(uri, cwd);
    if ((!fs_1.existsSync(filePath) && !create) || !fs_1.existsSync(path_1.default.dirname(filePath))) {
        throw new foundation_1.SystemError(foundation_1.SystemErrorNo.ENOENT);
    }
    if (fs_1.existsSync(filePath) && !overwrite) {
        throw new foundation_1.SystemError(foundation_1.SystemErrorNo.EEXIST);
    }
    return fs_1.promises.writeFile(filePath, base64Content, 'base64');
};
const deleteFile = ({ uri, options: { recursive } }, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const filePath = foundation_1.resolveUri(uri, cwd);
    if (!recursive) {
        const stats = yield fs_1.promises.stat(filePath);
        if (stats.isDirectory()) {
            throw new foundation_1.SystemError(foundation_1.SystemErrorNo.EISDIR);
        }
    }
    return fs_1.promises.rm(filePath, { recursive });
});
const renameOrCopy = (type, { oldUri, newUri, options: { overwrite } }, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const oldPath = foundation_1.resolveUri(oldUri, cwd);
    const newPath = foundation_1.resolveUri(newUri, cwd);
    if (!fs_1.existsSync(oldPath) || !fs_1.existsSync(path_1.default.dirname(newPath))) {
        throw new foundation_1.SystemError(foundation_1.SystemErrorNo.ENOENT);
    }
    if (fs_1.existsSync(newPath)) {
        if (!overwrite) {
            throw new foundation_1.SystemError(foundation_1.SystemErrorNo.EEXIST);
        }
        yield fs_1.promises.rm(newPath, { recursive: true });
    }
    return type === core_1.FSEventType.Copy
        ? fs_1.promises.copyFile(oldPath, newPath)
        : fs_1.promises.rename(oldPath, newPath);
});
const registerFSEventHandlers = (socket, cwd) => {
    socket.on(core_1.VscClientEvent.FSEvent, (uuid, type, payload) => __awaiter(void 0, void 0, void 0, function* () {
        // TO-DO: add error handling
        const emitResult = (data) => socket.emit(core_1.RunnerClientEvent.FSEvent, uuid, data);
        const emitError = (data) => socket.emit(core_1.RunnerClientEvent.FSEventError, uuid, data);
        try {
            if (type === core_1.FSEventType.Stat) {
                emitResult(yield stat(payload, cwd));
            }
            if (type === core_1.FSEventType.ReadDirectory) {
                emitResult(yield readDirectory(payload, cwd));
            }
            if (type === core_1.FSEventType.ReadFile) {
                emitResult(yield readFile(payload, cwd));
            }
            if (type === core_1.FSEventType.CreateDirectory) {
                emitResult(yield createDirectory(payload, cwd));
            }
            if (type === core_1.FSEventType.WriteFile) {
                emitResult(yield writeFile(payload, cwd));
            }
            if (type === core_1.FSEventType.Delete) {
                emitResult(yield deleteFile(payload, cwd));
            }
            if (type === core_1.FSEventType.Rename || type === core_1.FSEventType.Copy) {
                emitResult(yield renameOrCopy(type, payload, cwd));
            }
            if (type === core_1.FSEventType.FileSearch) {
                emitResult(yield search_1.fileSearch(payload, cwd));
            }
            if (type === core_1.FSEventType.TextSearch) {
                emitResult(yield search_1.textSearch(payload, cwd, (match) => socket.emit(core_1.RunnerClientEvent.FSTextSearchMatch, uuid, match)));
            }
        }
        catch (error) {
            logger_1.default.verbose('FS error');
            logger_1.default.verbose(JSON.stringify(error));
            emitError(error);
        }
    }));
};
exports.registerFSEventHandlers = registerFSEventHandlers;
//# sourceMappingURL=index.js.map